==> copilot-24.4.6-sources.jar: com/vaadin/copilot/ErrorHandler.java <==
package com.vaadin.copilot;

import com.vaadin.base.devserver.DevToolsInterface;
import com.vaadin.flow.internal.JsonUtils;
import elemental.json.Json;
import elemental.json.JsonArray;
import elemental.json.JsonObject;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.PrintWriter;
import java.io.StringWriter;
import java.util.Arrays;
import java.util.concurrent.atomic.AtomicBoolean;
import java.util.stream.Stream;

public class ErrorHandler {

    public static final String ERROR_KEY = "error";
    public static final String ERROR_EXCEPTION_MESSAGE_KEY = "errorMessage";
    public static final String ERROR_STACKTRACE_KEY = "errorStacktrace";

    public static JsonArray toJson(Exception e) {
        StringWriter sw = new StringWriter();
        try (PrintWriter pw = new PrintWriter(sw)) {
            e.printStackTrace(pw);
        }
        // We don't need the stack trace after
        // "com.vaadin.base.devserver.DebugWindowConnection.onMessage" as that
        // is outside Copilot
        // But we want all "causes" parts of the stack trace
        String[] stack = sw.toString().split("\n");

        AtomicBoolean include = new AtomicBoolean(true);
        Stream<String> filtered = Arrays.stream(stack).filter(s -> {
            if (s.contains(
                    "com.vaadin.base.devserver.DebugWindowConnection.onMessage")) {
                include.set(false);
            }
            if (!s.startsWith("\t")) {
                include.set(true);
            }
            return include.get();
        });
        return filtered.map(Json::create).collect(JsonUtils.asArray());
    }

    public static void setError(JsonObject respData, String error) {
        respData.put(ErrorHandler.ERROR_KEY, error);
    }

    public static void setError(JsonObject respData, String error,
            Exception e) {
        setError(respData, error);
        respData.put(ERROR_EXCEPTION_MESSAGE_KEY, e.getMessage());
        respData.put(ERROR_STACKTRACE_KEY, ErrorHandler.toJson(e));
        if (Copilot.isDevelopmentMode()) {
            getLogger().error("Stack trace because in Copilot development mode",
                    e);
        }
    }

    private static Logger getLogger() {
        return LoggerFactory.getLogger(ErrorHandler.class);
    }

    public static void sendErrorResponse(DevToolsInterface devToolsInterface,
            String command, JsonObject responseData, String error,
            Exception e) {
        ErrorHandler.setError(responseData, error, e);
        devToolsInterface.send(Copilot.PREFIX + command + "-response",
                responseData);
    }
}

==> copilot-24.4.20-sources.jar: com/vaadin/copilot/ErrorHandler.java <==
package com.vaadin.copilot;

import com.vaadin.base.devserver.DevToolsInterface;
import com.vaadin.flow.internal.JsonUtils;
import elemental.json.Json;
import elemental.json.JsonArray;
import elemental.json.JsonObject;
import java.io.PrintWriter;
import java.io.StringWriter;
import java.util.Arrays;
import java.util.concurrent.atomic.AtomicBoolean;
import java.util.stream.Stream;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class ErrorHandler {

    public static final String ERROR_KEY = "error";
    public static final String ERROR_EXCEPTION_MESSAGE_KEY = "errorMessage";
    public static final String ERROR_STACKTRACE_KEY = "errorStacktrace";

    public static JsonArray toJson(Exception e) {
        StringWriter sw = new StringWriter();
        try (PrintWriter pw = new PrintWriter(sw)) {
            e.printStackTrace(pw);
        }
        // We don't need the stack trace after
        // "com.vaadin.base.devserver.DebugWindowConnection.onMessage" as that
        // is outside Copilot
        // But we want all "causes" parts of the stack trace
        String[] stack = sw.toString().split("\n");

        AtomicBoolean include = new AtomicBoolean(true);
        Stream<String> filtered = Arrays.stream(stack).filter(s -> {
            if (s.contains("com.vaadin.base.devserver.DebugWindowConnection.onMessage")) {
                include.set(false);
            }
            if (!s.startsWith("\t")) {
                include.set(true);
            }
            return include.get();
        });
        return filtered.map(Json::create).collect(JsonUtils.asArray());
    }

    public static void setError(JsonObject respData, String error) {
        respData.put(ErrorHandler.ERROR_KEY, error);
    }

    public static void setError(JsonObject respData, String error, Exception e) {
        setError(respData, error);
        respData.put(ERROR_EXCEPTION_MESSAGE_KEY, e.getMessage());
        respData.put(ERROR_STACKTRACE_KEY, ErrorHandler.toJson(e));
        if (Copilot.isDevelopmentMode()) {
            getLogger().error("Stack trace because in Copilot development mode", e);
        }
    }

    private static Logger getLogger() {
        return LoggerFactory.getLogger(ErrorHandler.class);
    }

    public static void sendErrorResponse(
            DevToolsInterface devToolsInterface, String command, JsonObject responseData, String error, Exception e) {
        ErrorHandler.setError(responseData, error, e);
        devToolsInterface.send(Copilot.PREFIX + command + "-response", responseData);
    }
}
